/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "iskufxconfig_application.h"
#include "roccat_config_window.h"
#include "iskufxconfig_configuration_dialog.h"
#include "iskufxconfig_profile_page.h"
#include "isku_settings_page.h"
#include "roccat_warning_dialog.h"
#include "roccat_save_dialog.h"
#include "roccat_text_dialog.h"
#include "dbus_roccat_helper.h"

struct _IskufxconfigApplication {
	RoccatDevice *iskufx;
	RoccatDeviceScanner *device_scanner;
	RoccatConfigWindow *window;
	IskufxconfigProfilePage *profile_pages[ISKU_PROFILE_NUM];
	IskuSettingsPage *settings_page;
	RoccatKeyFile *config;
	DBusGProxy *dbus_proxy;
	guint actual_profile_index;
};

static gboolean warn_if_no_device(IskufxconfigApplication *app) {
	return roccat_no_device_warning_dialog(GTK_WINDOW(app->window), app->iskufx, ISKUFX_DEVICE_NAME);
}

static void settings_page_actual_profile_changed_cb(IskuSettingsPage *settings_page, gpointer user_data) {
	IskufxconfigApplication *app = (IskufxconfigApplication *)user_data;
	guint profile_number = isku_settings_page_get_actual_profile(app->settings_page);
	guint profile_index = profile_number - 1;
	GError *local_error = NULL;

	if (!warn_if_no_device(app))
		return;

	if (app->actual_profile_index != profile_index) {
		app->actual_profile_index = profile_index;
		isku_actual_profile_write(app->iskufx, profile_index, &local_error);
		if (!roccat_handle_error_dialog(GTK_WINDOW(app->window), "Error activating profile", &local_error))
			return;
		iskufx_dbus_emit_profile_changed_outside(app->dbus_proxy, profile_number);
	}
}

static void device_reset_cb(IskuSettingsPage *settings_page, gpointer user_data) {
	IskufxconfigApplication *app = (IskufxconfigApplication *)user_data;
	IskuRkp *rkp;
	guint i;
	GError *error = NULL;

	if (!warn_if_no_device(app))
		return;

	isku_reset(app->iskufx, ISKU_RESET_FUNCTION_RESET, &error);
	if (!roccat_handle_error_dialog(GTK_WINDOW(app->window), "Error resetting device", &error))
		return;

	for (i = 0; i < ISKU_PROFILE_NUM; ++i) {
		rkp = iskufx_rkp_load_save_after_reset(app->iskufx, i, &error);
		iskufxconfig_profile_page_set_rkp(app->profile_pages[i], rkp);
		isku_rkp_free(rkp);
		iskufx_dbus_emit_profile_data_changed_outside(app->dbus_proxy, i + 1);
	}
}

static void actual_profile_changed_from_device_cb(DBusGProxy *proxy, guchar profile_number, gpointer user_data) {
	IskufxconfigApplication *app = (IskufxconfigApplication *)user_data;
	guint profile_index = profile_number - 1;

	if (app->actual_profile_index != profile_index) {
		app->actual_profile_index = profile_index;
		isku_settings_page_set_actual_profile(app->settings_page, profile_number);
	}
}

static void brightness_changed_from_device_cb(DBusGProxy *proxy, guchar profile_number, guchar brightness_number, gpointer user_data) {
	IskufxconfigApplication *app = (IskufxconfigApplication *)user_data;
	iskufxconfig_profile_page_update_brightness(app->profile_pages[profile_number - 1], brightness_number - 1);
}

static void macro_changed_from_device_cb(DBusGProxy *proxy, guchar profile_number, guchar button_number, gchar const *macro, gpointer user_data) {
	IskufxconfigApplication *app = (IskufxconfigApplication *)user_data;
	IskuRkpMacroKeyInfo *key_info;
	gsize length;

	key_info = (IskuRkpMacroKeyInfo *)g_base64_decode(macro, &length);
	if (length == sizeof(IskuRkpMacroKeyInfo))
		iskufxconfig_profile_page_update_key(app->profile_pages[profile_number - 1], button_number - 1, key_info);
	g_free(key_info);
}

static gboolean settings_page_set_info(IskufxconfigApplication *app) {
	GError *local_error = NULL;
	IskuInfo *info;

	info = isku_info_read(app->iskufx, &local_error);
	if (!roccat_handle_error_dialog(GTK_WINDOW(app->window), "Can't read info:", &local_error))
		return FALSE;
	isku_settings_page_set_info(app->settings_page, info);
	g_free(info);
	return TRUE;
}

static gboolean add_cb(RoccatDevice *device, gpointer user_data) {
	IskufxconfigApplication *app = (IskufxconfigApplication *)user_data;
	GError *error = NULL;
	guint i;
	IskuRkp *rkp;

	if (app->iskufx) {
		roccat_multiple_device_warning_dialog(GTK_WINDOW(app->window), ISKUFX_DEVICE_NAME);
		return TRUE;
	}

	app->iskufx = device;

	app->actual_profile_index = isku_actual_profile_read(app->iskufx, &error);
	if (!roccat_handle_error_dialog(GTK_WINDOW(app->window), "Can't read actual profile:", &error))
		goto error;
	isku_settings_page_set_actual_profile(app->settings_page, app->actual_profile_index + 1);

	if (!settings_page_set_info(app))
		goto error;

	for (i = 0; i < ISKU_PROFILE_NUM; ++i) {
		rkp = iskufxconfig_profile_page_get_rkp(app->profile_pages[i]);
		iskufx_rkp_update_from_device(app->iskufx, rkp, i);
		iskufxconfig_profile_page_set_rkp(app->profile_pages[i], rkp);
		isku_rkp_free(rkp);
	}

	dbus_g_proxy_connect_signal(app->dbus_proxy, "ProfileChanged", G_CALLBACK(actual_profile_changed_from_device_cb), app, NULL);
	dbus_g_proxy_connect_signal(app->dbus_proxy, "BrightnessChanged", G_CALLBACK(brightness_changed_from_device_cb), app, NULL);
	dbus_g_proxy_connect_signal(app->dbus_proxy, "MacroChanged", G_CALLBACK(macro_changed_from_device_cb), app, NULL);

	roccat_config_window_set_title_connected(app->window, ISKUFX_DEVICE_NAME);

	return FALSE;
error:
	app->iskufx = NULL;
	return TRUE;
}

static gboolean remove_cb(RoccatDevice *device, gpointer user_data) {
	IskufxconfigApplication *app = (IskufxconfigApplication *)user_data;

	dbus_g_proxy_disconnect_signal(app->dbus_proxy, "ProfileChanged", G_CALLBACK(actual_profile_changed_from_device_cb), app);
	dbus_g_proxy_disconnect_signal(app->dbus_proxy, "BrightnessChanged", G_CALLBACK(brightness_changed_from_device_cb), app);
	dbus_g_proxy_disconnect_signal(app->dbus_proxy, "MacroChanged", G_CALLBACK(macro_changed_from_device_cb), app);

	roccat_device_free(app->iskufx);
	app->iskufx = NULL;

	isku_settings_page_clear_info(app->settings_page);
	roccat_config_window_set_title_disconnected(app->window);

	return TRUE;
}

static gboolean save_unsaved(IskufxconfigApplication *app, IskuRkp *rkp, guint profile_index, GError **error) {
	guint profile_number = profile_index + 1;

	if (app->iskufx)
		iskufx_rkp_save(app->iskufx, rkp, profile_index, error);
	else
		iskufx_rkp_save_actual(rkp, profile_index, error);

	if (*error)
		return FALSE;
	else {
		iskufx_dbus_emit_profile_data_changed_outside(app->dbus_proxy, profile_number);
		iskufxconfig_profile_page_set_rkp(app->profile_pages[profile_index], rkp);
		return TRUE;
	}
}

static gboolean display_quit_dialog(IskufxconfigApplication *app) {
	guint i;
	gboolean modified;
	IskuRkp *rkps[ISKU_PROFILE_NUM];
	gboolean retval;
	GError *error = NULL;

	modified = FALSE;
	for (i = 0; i < ISKU_PROFILE_NUM; ++i) {
		rkps[i] = iskufxconfig_profile_page_get_rkp(app->profile_pages[i]);
		if (isku_rkp_get_modified(rkps[i]))
			modified = TRUE;
	}
	if (!modified) {
		for (i = 0; i < ISKU_PROFILE_NUM; ++i)
			isku_rkp_free(rkps[i]);
		return TRUE;
	}

	switch (roccat_save_unsaved_dialog(GTK_WINDOW(app->window), TRUE)) {
	case GTK_RESPONSE_CANCEL:
		retval = FALSE;
		break;
	case GTK_RESPONSE_ACCEPT:
		retval = TRUE;
		warn_if_no_device(app);
		for (i = 0; i < ISKU_PROFILE_NUM; ++i) {
			if (!save_unsaved(app, rkps[i], i, &error)) {
				retval = FALSE;
				break;
			}
		}
		roccat_handle_error_dialog(GTK_WINDOW(app->window), "Error storing unsaved data", &error);
		break;
	default:
		retval = TRUE;
		break;
	}

	for (i = 0; i < ISKU_PROFILE_NUM; ++i)
		isku_rkp_free(rkps[i]);

	return retval;
}

static gboolean delete_event_cb(GtkWidget *window, GdkEvent *event, gpointer user_data) {
	IskufxconfigApplication *app = (IskufxconfigApplication *)user_data;

	return !display_quit_dialog(app);
}

static void destroy_event_cb(GtkWidget *window, gpointer user_data) {
	IskufxconfigApplication *app = (IskufxconfigApplication *)user_data;
	gtk_main_quit();
	app->window = NULL;
}

static void save_profile_to_device_cb(IskufxconfigProfilePage *profile_page, gpointer user_data) {
	IskufxconfigApplication *app = (IskufxconfigApplication *)user_data;
	IskuRkp *rkp = iskufxconfig_profile_page_get_rkp(profile_page);
	guint profile_index = iskufxconfig_profile_page_get_index(profile_page);
	GError *error = NULL;

	warn_if_no_device(app);
	save_unsaved(app, rkp, profile_index, &error);
	roccat_handle_error_dialog(GTK_WINDOW(app->window), "Error storing unsaved data", &error);

	isku_rkp_free(rkp);
}

static void add_file_filter_to_file_chooser(GtkFileChooser *chooser) {
	GtkFileFilter *filter;
	filter = gtk_file_filter_new();
	gtk_file_filter_set_name(filter, "IskuFX Windows profiles");
	gtk_file_filter_add_pattern(filter, "*.rkp");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(chooser), filter);

	filter = gtk_file_filter_new();
	gtk_file_filter_set_name(filter, "All files");
	gtk_file_filter_add_pattern(filter, "*");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(chooser), filter);
}

static void save_profile_to_file_cb(IskufxconfigProfilePage *profile_page, gpointer user_data) {
	IskufxconfigApplication *app = (IskufxconfigApplication *)user_data;
	GtkWidget *dialog;
	gchar *filename, *path;
	GError *error = NULL;
	guint profile_index = iskufxconfig_profile_page_get_index(profile_page);
	IskuRkp *rkp;

	dialog = gtk_file_chooser_dialog_new("Save profile",
			GTK_WINDOW(app->window),
			GTK_FILE_CHOOSER_ACTION_SAVE,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
			NULL);

	add_file_filter_to_file_chooser(GTK_FILE_CHOOSER(dialog));
	path = isku_configuration_get_rkp_path(app->config);
	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), path);
	g_free(path);

	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		path = g_path_get_dirname(filename);
		isku_configuration_set_rkp_path(app->config, path);
		g_free(path);

		rkp = iskufxconfig_profile_page_get_rkp(app->profile_pages[profile_index]);
		isku_rkp_write_with_path(filename, rkp, &error);
		isku_rkp_free(rkp);
		if (error) {
			roccat_warning_dialog(GTK_WINDOW(dialog), "Error saving rkp", error->message);
			g_error_free(error);
		}

		g_free(filename);
	}
	gtk_widget_destroy(dialog);
}

static void load_profile_from_file_cb(IskufxconfigProfilePage *profile_page, gpointer user_data) {
	IskufxconfigApplication *app = (IskufxconfigApplication *)user_data;
	GtkWidget *dialog;
	gchar *filename, *path;
	GError *error = NULL;
	IskuRkp *rkp;
	guint profile_index = iskufxconfig_profile_page_get_index(profile_page);

	dialog = gtk_file_chooser_dialog_new("Load profile",
			GTK_WINDOW(app->window),
			GTK_FILE_CHOOSER_ACTION_OPEN,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
			NULL);

	add_file_filter_to_file_chooser(GTK_FILE_CHOOSER(dialog));
	path = isku_configuration_get_rkp_path(app->config);
	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), path);
	g_free(path);

	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		path = g_path_get_dirname(filename);
		isku_configuration_set_rkp_path(app->config, path);
		g_free(path);

		rkp = isku_rkp_read_with_path(filename, &error);
		if (error) {
			roccat_warning_dialog(GTK_WINDOW(dialog), "Error loading rkp", error->message);
			g_error_free(error);
		} else {
			isku_rkp_set_modified(rkp);
			iskufxconfig_profile_page_set_rkp(app->profile_pages[profile_index], rkp);
			isku_rkp_free(rkp);
		}

		g_free(filename);
	}

	gtk_widget_destroy(dialog);
}

static void save_all_cb(GtkMenuItem *item, gpointer user_data) {
	IskufxconfigApplication *app = (IskufxconfigApplication *)user_data;
	IskuRkp *rkp;
	guint i;
	GError *error = NULL;

	warn_if_no_device(app);
	for (i = 0; i < ISKU_PROFILE_NUM; ++i) {
		rkp = iskufxconfig_profile_page_get_rkp(app->profile_pages[i]);
		save_unsaved(app, rkp, i, &error);
		roccat_handle_error_dialog(GTK_WINDOW(app->window), "Error storing unsaved data", &error);

		isku_rkp_free(rkp);
	}
}

static void edit_preferences_cb(GtkMenuItem *item, gpointer user_data) {
	IskufxconfigApplication *app = (IskufxconfigApplication *)user_data;
	iskufxconfig_configuration_dialog(GTK_WINDOW(app->window), app->config);
	iskufx_dbus_emit_configuration_changed_outside(app->dbus_proxy);
}

IskufxconfigApplication *iskufxconfig_application_new(void) {
	IskufxconfigApplication *app;
	IskuRkp *rkp;
	GtkWidget *tab_label;
	guint i;
	GtkMenuItem *preferences_menu;

	app = (IskufxconfigApplication *)g_malloc0(sizeof(IskufxconfigApplication));

	app->config = iskufx_configuration_load();

	app->window = ROCCAT_CONFIG_WINDOW(roccat_config_window_new(ISKUFX_DEVICE_NAME));
	gtk_widget_show(GTK_WIDGET(app->window));

	for (i = 0; i < ISKU_PROFILE_NUM; ++i) {
		app->profile_pages[i] = ISKUFXCONFIG_PROFILE_PAGE(iskufxconfig_profile_page_new(i));
		tab_label = iskufxconfig_profile_page_get_tab_label(app->profile_pages[i]);
		roccat_config_window_append_page(app->window, GTK_WIDGET(app->profile_pages[i]), tab_label);

		rkp = iskufx_rkp_load_actual(i);
		iskufxconfig_profile_page_set_rkp(app->profile_pages[i], rkp);
		isku_rkp_free(rkp);

		g_signal_connect(G_OBJECT(app->profile_pages[i]), "load-from-file", G_CALLBACK(load_profile_from_file_cb), app);
		g_signal_connect(G_OBJECT(app->profile_pages[i]), "save-to-file", G_CALLBACK(save_profile_to_file_cb), app);
		g_signal_connect(G_OBJECT(app->profile_pages[i]), "save-to-device", G_CALLBACK(save_profile_to_device_cb), app);
	}

	app->settings_page = ISKU_SETTINGS_PAGE(isku_settings_page_new());
	roccat_config_window_append_page(app->window, GTK_WIDGET(app->settings_page), gtk_label_new("Settings"));

	preferences_menu = GTK_MENU_ITEM(gtk_image_menu_item_new_from_stock(GTK_STOCK_PREFERENCES, NULL));
	g_signal_connect(G_OBJECT(preferences_menu), "activate", G_CALLBACK(edit_preferences_cb), app);
	roccat_config_window_edit_menu_append(app->window, preferences_menu);

	g_signal_connect(G_OBJECT(app->settings_page), "actual-profile-changed", G_CALLBACK(settings_page_actual_profile_changed_cb), app);
	g_signal_connect(G_OBJECT(app->settings_page), "device-reset", G_CALLBACK(device_reset_cb), app);
	g_signal_connect(G_OBJECT(app->window), "destroy", G_CALLBACK(destroy_event_cb), app);
	g_signal_connect(G_OBJECT(app->window), "delete-event", G_CALLBACK(delete_event_cb), app);
	g_signal_connect(G_OBJECT(app->window), "save-all", G_CALLBACK(save_all_cb), app);

	/* keep this order */
	app->dbus_proxy = iskufx_dbus_proxy_new();
	app->device_scanner = iskufx_device_scanner_new(add_cb, remove_cb, app);

	/* if a device is already plugged in, add_cb has already been executed */
	warn_if_no_device(app);

	return app;
}

void iskufxconfig_application_free(IskufxconfigApplication *app) {
	roccat_device_scanner_free(app->device_scanner);
	app->device_scanner = NULL;

	dbus_roccat_proxy_free(app->dbus_proxy);
	app->dbus_proxy = NULL;

	roccat_device_free(app->iskufx);
	app->iskufx = NULL;

	if (app->window) {
		gtk_widget_destroy(GTK_WIDGET(app->window));
		app->window = NULL;
	}

	iskufxconfig_configuration_save(GTK_WINDOW(app->window), app->config);
	isku_configuration_free(app->config);
	app->config = NULL;

	g_free(app);
}
